# Stubs for xml.etree.ElementInclude (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

XINCLUDE = ... # type: Any
XINCLUDE_INCLUDE = ... # type: Any
XINCLUDE_FALLBACK = ... # type: Any

class FatalIncludeError(SyntaxError): ...

def default_loader(href, parse, encoding=...): ...
def include(elem, loader=...): ...
