# Stubs for Queue (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class Empty(Exception): ...
class Full(Exception): ...

class Queue:
    maxsize = ... # type: Any
    mutex = ... # type: Any
    not_empty = ... # type: Any
    not_full = ... # type: Any
    all_tasks_done = ... # type: Any
    unfinished_tasks = ... # type: Any
    def __init__(self, maxsize: int = ...) -> None: ...
    def task_done(self) -> None: ...
    def join(self) -> None: ...
    def qsize(self) -> int: ...
    def empty(self) -> bool: ...
    def full(self) -> bool: ...
    def put(self, item: Any, block: bool = ..., timeout: float = ...) -> None: ...
    def put_nowait(self, item) -> None: ...
    def get(self, block: bool = ..., timeout: float = ...) -> Any: ...
    def get_nowait(self) -> Any: ...

class PriorityQueue(Queue): ...
class LifoQueue(Queue): ...
