# Stubs for xml.etree.ElementPath (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

xpath_tokenizer_re = ... # type: Any

def xpath_tokenizer(pattern, namespaces=...): ...
def get_parent_map(context): ...
def prepare_child(next, token): ...
def prepare_star(next, token): ...
def prepare_self(next, token): ...
def prepare_descendant(next, token): ...
def prepare_parent(next, token): ...
def prepare_predicate(next, token): ...

ops = ... # type: Any

class _SelectorContext:
    parent_map = ... # type: Any
    root = ... # type: Any
    def __init__(self, root) -> None: ...

def iterfind(elem, path, namespaces=...): ...
def find(elem, path, namespaces=...): ...
def findall(elem, path, namespaces=...): ...
def findtext(elem, path, default=..., namespaces=...): ...
